// Power supply based upon a Laptop 19V power brick and an Amazon Buck-Boost switchin power supply
// MCU is RP2040 Pico board
// Good up to about 3.8A @ 23.5 volts
// Good up to 8A or more at 4 volts



#include "ADS1015_BM.h" // This is a BM-modified version of adafruit_AD1015 library
#include <max7219.h>
#include <floatToString.h>
#include <PicoAnalogCorrection.h>
#include "pio_encoder.h"

ADS1115_BM ads;  /* Use this for the 16-bit version */
MAX7219 max7219;  // by default uses GPIO10 for CLK, 11 for CS, 12 for DIN

const uint8_t GND_PIN = A1;  // GND meas pin
const uint8_t VCC_PIN = A2;  // VCC meas pin
const uint8_t ADC_RES = 12;  // ADC bits
const float VREF = 3.256;      // Analog reference voltage

PioEncoder encoder(2); // encoder is connected to GPIO2 and GPIO3

PicoAnalogCorrection pico(ADC_RES, VREF);

  String temp;
  char temp2[8];
  int y;
  int currentLimitComparator= 2768;   // = 1 Amp
  bool powerOn= false;

void setup(void)
{
  pinMode(4,INPUT_PULLUP); // Rot encoder Enter sw
  pinMode(13,INPUT_PULLUP);
  pinMode(14,INPUT); // ADS1115 Comparator output monitor
  pinMode(15,OUTPUT);   // MOSFET GATE to SMPS
  digitalWrite(15,LOW);  // Power off for now
  Serial.begin(9600);
  max7219.Begin();
  encoder.begin();
  Serial.println("RP2040 Power Supply");
  analogReadResolution(12);
  pico.calibrateAdc(GND_PIN, VCC_PIN, 5000);
  pico.returnCalibrationValues();
  Serial.print("Uncalibrated GND: ");
  Serial.println(pico.analogRead(GND_PIN));
  Serial.print("Calibrated GND: ");
  Serial.println(pico.analogCRead(GND_PIN, 10));
  Serial.print("Uncalibrated VCC: ");
  Serial.println(pico.analogRead(VCC_PIN));
  Serial.print("Calibrated VCC: ");
  Serial.println(pico.analogCRead(VCC_PIN, 10));
   
 
  Wire.setSDA(8);  // use GPIO8 for SDA on RP2040
  Wire.setSCL(9);  // Use GPIO9 for SCL on RP2040 
  max7219.Clear();

  ads.begin();
  ads.setGain(GAIN_SIXTEEN);
 	ads.startComparator_Differential_0_1(ADS1115_REG_CONFIG_DR_128SPS,  currentLimitComparator);  // this sets ADC channel to Differential ch 0,1
  
  int a;
  do {
  int16_t	adc0 = ads.getLastConversionResults(128);  // Read ADC until  Comparator flag is cleared
  a = digitalRead(14);
  }
 while (a == 0);
}

void loop(void)
{
    // check for rot. encoder pushed
  if (digitalRead(4) ==0)  {
    delay(50);
    if (digitalRead(4) == 0) {
      readEncoder();
    }
  }

  int v= pico.analogCRead(26,100);
  float voltage = (float)v /100.0;  
//  Serial.print(" ADC= Volts ");
//  Serial.print(voltage);
  int16_t adc0, adc1;
  float current;

 if (digitalRead(14)== LOW) 
 { 
   digitalWrite(15,LOW);   // MOSFET gate is active low
   Serial.println("tripped");
   powerOn=false;
 }
   adc0 = ads.getLastConversionResults(128);  
   current= (float) adc0/2768;
//  Serial.print("ADC reading= "); 
//  Serial.println(adc0);
  floatToString(voltage,temp2,sizeof(temp2),2);
  temp.toCharArray(temp2, temp.length());
  max7219.DisplayText(temp2, 0); //0=left justify 1=right justify
  floatToString(current,temp2,sizeof(temp2),2);
  temp.toCharArray(temp2, temp.length());
  max7219.DisplayText(temp2, 1); //0=left justify 1=right justify
  // check for rotary encoder enter button pushed (or power resert button and skip delay if so
  for (int i=0;i<50;i++){
      if (digitalRead(4) ==0) exit;

      if ((digitalRead(13) ==0) && (!powerOn)) {
          digitalWrite (15,HIGH); // Restore power to the SMPS
          powerOn=true; 
          delay(100);
          while(digitalRead(13) ==0) ;;;  //wait til button released
          delay(100);  //debounce  
          int a; 
          do {
           int16_t	adc0 = ads.getLastConversionResults(128);  // Read ADC once to clear the Comparator flag
           a = digitalRead(14);
          } while (a == 0);    
      }
     if ((digitalRead(13) ==0) && (powerOn)) {
          digitalWrite (15,LOW); // Remove power to the SMPS
          powerOn= false; 
          delay(100);
          while(digitalRead(13) ==0) ;;;  //wait til button released
          delay(100);  //debounce         
     }  
      delay(10);
  }
    max7219.Clear();
    Serial.print("poweron= ");
    Serial.println(powerOn);
}


void readEncoder() {
 max7219.Clear();
 delay(50);
  while (digitalRead(4) ==0) {
    delay(10);
  }
  delay(200);
  while(1) {
    int a= encoder.getCount();
    if (a <0) {
      a=0;
      encoder.reset();
    }
    float currentLimit = (float) a/80;
    float t= currentLimit *2768;
    currentLimitComparator = (int) t;
    Serial.println(currentLimitComparator);
    floatToString(currentLimit,temp2,sizeof(temp2),2);
    temp.toCharArray(temp2, temp.length());
    max7219.DisplayText(temp2, 1); //0=left justify 1=right justify
    delay(100);
    max7219.Clear();
   if (digitalRead(4) ==0) {
    ads.startComparator_Differential_0_1(ADS1115_REG_CONFIG_DR_128SPS,  currentLimitComparator);
    return; 
   } 
  }
 
}